package data.scripts.wyv;

import java.awt.Color;

import com.fs.starfarer.api.Global;
import com.fs.starfarer.api.campaign.PlanetAPI;
import com.fs.starfarer.api.campaign.econ.CommoditySpecAPI;
import com.fs.starfarer.api.impl.campaign.ids.Stats;
import com.fs.starfarer.api.impl.campaign.ids.Strings;
import com.fs.starfarer.api.ui.TooltipMakerAPI;
import com.fs.starfarer.api.util.Misc;
import com.fs.starfarer.api.impl.campaign.econ.impl.BaseIndustry;
import com.fs.starfarer.api.campaign.econ.MarketAPI;
import com.fs.starfarer.api.impl.campaign.population.PopulationComposition;
import com.fs.starfarer.api.impl.campaign.ids.Commodities;
import com.fs.starfarer.api.campaign.econ.MarketImmigrationModifier;


public class PlanetaryShield extends BaseIndustry implements MarketImmigrationModifier {

	private static final String BASE_DESC = "An envelope of charged plasma seeded with exotic virtual particles producing effects which disrupt the combat effectiveness of long-range bombardment and even incursion by small vessels. The planetary shield can be continually charged and controlled by a central source and presents a formidable obstacle to any form of ground assault.";
	private static final String MOD_DESC = BASE_DESC + "\n\nIt also presents a formidable obstacle to any unscheduled transfer of goods or personnel, reducing colony accessibility, population growth, and - if the colony isn't a free port - the odds of pather terrorists gaining a foothold on the ground.";
	private static final String METEOR_DESC = MOD_DESC + "\n\nAnd, last but not least, it offers almost-perfect protection against the planet's frequent meteor impacts - though there are still significant maintenance costs incurred to ensure that a shield failure would not prove to be a colony-ending disaster.";
	
	public void setNewBuild( float progress ) {
		building = true;
		buildProgress = spec.getBuildTime() * progress;
		upgradeId = null;
		buildTime = spec.getBuildTime();
	}
	
	public void apply() {
		super.apply(false);
		
		int size = 5;
		applyIncomeAndUpkeep(size);
		
		if( PSModPlugin.revertToVanilla ) {
			market.getAccessibilityMod().unmodifyFlat( getModId(0) );
			market.getStats().getDynamic().getMod(Stats.GROUND_DEFENSES_MOD).unmodifyFlat( getModId() );
			market.getStats().getDynamic().getMod(Stats.GROUND_DEFENSES_MOD).modifyMult( getModId(), 3f, getNameForModifier() );
			market.getHazard().unmodifyFlat(getModId());
		} else {
			market.getAccessibilityMod().modifyFlat( getModId(0), -0.1f, getNameForModifier() );
			market.getStats().getDynamic().getMod(Stats.GROUND_DEFENSES_MOD).unmodifyMult( getModId() );
			market.getStats().getDynamic().getMod(Stats.GROUND_DEFENSES_MOD).modifyFlat( getModId(), 200f, getNameForModifier() );
			if( null != market.getFirstCondition( "meteor_impacts" ) ) {
				market.getHazard().modifyFlat(getModId(), -0.25f, getNameForModifier());
			} else {
				market.getHazard().unmodifyFlat( getModId() );
			}
		}
		
		if (isFunctional()) {
			applyVisuals(market.getPlanetEntity());
		} else {
			unapply();
		}
	}
	
	public static void applyVisuals(PlanetAPI planet) {
		if (planet == null) return;
		planet.getSpec().setShieldTexture(Global.getSettings().getSpriteName("industry", "shield_texture"));
		planet.getSpec().setShieldThickness(0.1f);
		planet.getSpec().setShieldColor(new Color(255,255,255,175));
		planet.applySpecChanges();
	}
	
	public static void unapplyVisuals(PlanetAPI planet) {
		if (planet == null) return;
		planet.getSpec().setShieldTexture(null);
		planet.getSpec().setShieldThickness(0f);
		planet.getSpec().setShieldColor(null);
		planet.applySpecChanges();
	}

	
	@Override
	public void unapply() {
		super.unapply();
		
		market.getHazard().unmodifyFlat( getModId() );
		market.getAccessibilityMod().unmodifyFlat( getModId( 0 ) );
		
		unapplyVisuals(market.getPlanetEntity());
		
		market.getStats().getDynamic().getMod(Stats.GROUND_DEFENSES_MOD).unmodifyMult(getModId());
		market.getStats().getDynamic().getMod(Stats.GROUND_DEFENSES_MOD).unmodifyFlat(getModId());
	}

	@Override
	public boolean isAvailableToBuild() {
		if (!Global.getSector().getPlayerFaction().knowsIndustry(getId())) {
			return false;
		}
		return market.getPlanetEntity() != null;
	}
	
	public boolean showWhenUnavailable() {
		return Global.getSector().getPlayerFaction().knowsIndustry(getId());
	}

	protected boolean hasPostDemandSection(boolean hasDemand, IndustryTooltipMode mode) {
		return mode != IndustryTooltipMode.NORMAL || isFunctional();
	}
	
	@Override
	protected void addPostDemandSection(TooltipMakerAPI tooltip, boolean hasDemand, IndustryTooltipMode mode) {
		if (mode != IndustryTooltipMode.NORMAL || isFunctional()) {
			Color h = Misc.getHighlightColor();
			float opad = 10f;
			
			if( PSModPlugin.revertToVanilla ) {
				tooltip.addPara( "Ground defense strength: %s", opad, h, "x3" );
			} else {
				tooltip.addPara("Accessibility penalty: %s", opad, h, "-10%");
				tooltip.addPara( "Population growth: %s", opad, h, "" + getPopPenalty() );
				tooltip.addPara( "Ground defense strength: %s", opad, h, "+200" );
				if( null != market.getFirstCondition( "meteor_impacts" ) ) {
					tooltip.addPara( "Hazard rating: %s", opad, h, "-25%" );
				}
			}
		}
	}
	
	
	@Override
	protected void applyAlphaCoreModifiers() {
		if( PSModPlugin.revertToVanilla ) {
			market.getStats().getDynamic().getMod(Stats.GROUND_DEFENSES_MOD).unmodifyFlat( getModId(1) );
			market.getStats().getDynamic().getMod(Stats.GROUND_DEFENSES_MOD).modifyMult( getModId(1), 1.5f, "Alpha core (" + getNameForModifier() + ")");
		} else {
			market.getStats().getDynamic().getMod(Stats.GROUND_DEFENSES_MOD).unmodifyMult( getModId(1) );
			market.getStats().getDynamic().getMod(Stats.GROUND_DEFENSES_MOD).modifyFlat( getModId(1), 100f, "Alpha core (" + getNameForModifier() + ")");
		}
	}
	
	@Override
	protected void applyBetaCoreModifiers() {
		market.getStats().getDynamic().getMod(Stats.GROUND_DEFENSES_MOD).unmodifyFlat( getModId(1) );
		market.getStats().getDynamic().getMod(Stats.GROUND_DEFENSES_MOD).unmodifyMult( getModId(1) );
	}
	
	@Override
	protected void applyGammaCoreModifiers() {
		market.getStats().getDynamic().getMod(Stats.GROUND_DEFENSES_MOD).unmodifyFlat( getModId(1) );
		market.getStats().getDynamic().getMod(Stats.GROUND_DEFENSES_MOD).unmodifyMult( getModId(1) );
	}
	
	@Override
	protected void applyNoAICoreModifiers() {
		market.getStats().getDynamic().getMod(Stats.GROUND_DEFENSES_MOD).unmodifyFlat( getModId(1) );
		market.getStats().getDynamic().getMod(Stats.GROUND_DEFENSES_MOD).unmodifyMult( getModId(1) );
	}
	
	//@Override
	//protected void applyAlphaCoreSupplyAndDemandModifiers() {
	//	demandReduction.modifyFlat(getModId(0), DEMAND_REDUCTION, "Alpha core");
	//}
	
	@Override
	protected void addAlphaCoreDescription(TooltipMakerAPI tooltip, AICoreDescriptionMode mode) {
		float opad = 10f;
		Color highlight = Misc.getHighlightColor();
		
		String pre = "Alpha-level AI core currently assigned. ";
		if (mode == AICoreDescriptionMode.MANAGE_CORE_DIALOG_LIST || mode == AICoreDescriptionMode.INDUSTRY_TOOLTIP) {
			pre = "Alpha-level AI core. ";
		}
		String modifierString;
		if( PSModPlugin.revertToVanilla ) {
			modifierString = "x1.5";
		} else {
			modifierString = "+100";
		}
		
		if (mode == AICoreDescriptionMode.INDUSTRY_TOOLTIP) {
			CommoditySpecAPI coreSpec = Global.getSettings().getCommoditySpec(aiCoreId);
			TooltipMakerAPI text = tooltip.beginImageWithText(coreSpec.getIconName(), 48);
			text.addPara(pre + "Reduces upkeep cost by %s. Reduces demand by %s unit. " +
					"Increases ground defenses by %s.", 0f, highlight,
					"" + (int)((1f - UPKEEP_MULT) * 100f) + "%", "" + DEMAND_REDUCTION,
					modifierString );
			tooltip.addImageWithText(opad);
			return;
		}
		
		tooltip.addPara(pre + "Reduces upkeep cost by %s. Reduces demand by %s unit. " +
				"Increases ground defenses by %s.", opad, highlight,
				"" + (int)((1f - UPKEEP_MULT) * 100f) + "%", "" + DEMAND_REDUCTION,
				modifierString );
		
	}
	
	@Override
	protected String getDescriptionOverride() {
		if( PSModPlugin.revertToVanilla ) {
			return BASE_DESC;
		} else if( null != market.getFirstCondition( "meteor_impacts" ) ) {
			return METEOR_DESC;
		} else {
			return MOD_DESC;
		}
	}

	@Override
	public float getPatherInterest() {
		if( PSModPlugin.revertToVanilla ) return super.getPatherInterest();
		if( !isFunctional() || getMarket().isFreePort() ) return 4f + super.getPatherInterest();
		if (Commodities.ALPHA_CORE.equals(aiCoreId)) {
			return -34f + super.getPatherInterest(); // this should always be 30.
		}
		return -9f + super.getPatherInterest(); // why you'd install a beta or gamma core in a planetary shield generator I don't know, but we should account for it anyway.
	}
    
	@Override
	public void modifyIncoming(MarketAPI market, PopulationComposition incoming) {
		if( isFunctional() && !PSModPlugin.revertToVanilla ) {
			incoming.getWeight().modifyFlat( getModId(), getPopPenalty(), getNameForModifier() );
		} else {
			incoming.getWeight().unmodifyFlat( getModId() );
		}
	}
	
	private float getPopPenalty() {
		int size = getMarket().getSize();
		return -22f + ( 2f * size ); // -14 for a size three colony, scaling to -2 for a size 10 colony... where, okay, it doesn't matter anymore.
	}
	
}
